@@ -395,7 +395,6 @@ def found_next_open_segment_point(mpos, objects, max_dist=None, exclude=None):
             for segmentd_idx in (0, -1):
                 if exclude and exclude[0] == obj_idx and exclude[1] == segmentd_idx:
                     continue
-
                 if segmentd_idx == 0:
                     pos_x = obj["segments"][segmentd_idx]["start"][0]
                     pos_y = obj["segments"][segmentd_idx]["start"][1]
@@ -408,7 +407,6 @@ def found_next_open_segment_point(mpos, objects, max_dist=None, exclude=None):
                 if min_dist is None or dist < min_dist:
                     min_dist = dist
                     nearest = (pos_x, pos_y, obj_idx, segmentd_idx)
-
     return nearest
 
 
@@ -865,10 +863,11 @@ def objects2polyline_offsets(diameter, objects, max_outer, small_circles=False):
 # analyze size
 def objects2minmax(objects):
     """find the min/max values of objects"""
-    min_x = objects[0]["segments"][0]["start"][0]
-    min_y = objects[0]["segments"][0]["start"][1]
-    max_x = objects[0]["segments"][0]["start"][0]
-    max_y = objects[0]["segments"][0]["start"][1]
+    fist_key = list(objects.keys())[0]
+    min_x = objects[fist_key]["segments"][0]["start"][0]
+    min_y = objects[fist_key]["segments"][0]["start"][1]
+    max_x = objects[fist_key]["segments"][0]["start"][0]
+    max_y = objects[fist_key]["segments"][0]["start"][1]
     for obj in objects.values():
         if obj.get("layer", "").startswith("BREAKS:") or obj.get(
             "layer", ""
