@@ -2,17 +2,18 @@
 
 import argparse
 import gettext
+import importlib
 import json
 import math
 import os
 import re
 import sys
-import tempfile
 from copy import deepcopy
 from functools import partial
 from pathlib import Path
-from typing import Union
+from typing import Optional, Union
 
+import ezdxf
 import setproctitle
 from PyQt5.QtGui import (  # pylint: disable=E0611
     QIcon,
@@ -70,11 +71,7 @@ from .gldraw import (
     draw_object_faces,
     draw_object_ids,
 )
-from .input_plugins.dxfread import DxfReader
-from .input_plugins.hpglread import HpglReader
-from .input_plugins.stlread import StlReader
-from .input_plugins.svgread import SvgReader
-from .input_plugins.ttfread import TtfReader
+from .input_plugins_base import DrawReaderBase
 from .machine_cmd import polylines2machine_cmd
 from .output_plugins.gcode_linuxcnc import PostProcessorGcodeLinuxCNC
 from .output_plugins.hpgl import PostProcessorHpgl
@@ -88,6 +85,17 @@ except ImportError:
     sys.exit(1)
 
 
+reader_plugins: dict = {}
+for reader in ("dxfread", "hpglread", "stlread", "svgread", "ttfread"):
+    try:
+        drawing_reader = importlib.import_module(
+            f".{reader}", "viaconstructor.input_plugins"
+        )
+        reader_plugins[reader] = drawing_reader.DrawReader
+    except Exception as reader_error:  # pylint: disable=W0703
+        print(f"ERRO while loading input plugin {reader}: {reader_error}")
+
+
 LAYER_REGEX = re.compile(r"([a-zA-Z]{1,4}):\s*([+-]?([0-9]+([.][0-9]*)?|[.][0-9]+))")
 
 
@@ -138,9 +146,7 @@ class GLWidget(QGLWidget):
     mpos = None
     mouse_pos_x = 0
     mouse_pos_y = 0
-    tab_selector = False
-    start_selector = False
-    repair_selector = False
+    selector_mode = ""
     selection = ()
     size_x = 0
     size_y = 0
@@ -222,7 +228,7 @@ class GLWidget(QGLWidget):
         GL.glCallList(self.project["gllist"])
 
         if self.selection:
-            if self.start_selector:
+            if self.selector_mode == "start":
                 # depth = self.project["setup"]["mill"]["depth"] - 0.1
                 depth = 0.1
                 GL.glLineWidth(5)
@@ -233,7 +239,7 @@ class GLWidget(QGLWidget):
                 GL.glVertex3f(self.selection[0] - 1, self.selection[1] + 1, depth)
                 GL.glVertex3f(self.selection[0] + 1, self.selection[1] - 1, depth)
                 GL.glEnd()
-            elif self.repair_selector:
+            elif self.selector_mode == "repair":
                 if len(self.selection) > 4:
                     depth = 0.1
                     GL.glLineWidth(15)
@@ -242,7 +248,16 @@ class GLWidget(QGLWidget):
                     GL.glVertex3f(self.selection[0], self.selection[1], depth)
                     GL.glVertex3f(self.selection[4], self.selection[5], depth)
                     GL.glEnd()
-
+            elif self.selector_mode == "delete":
+                depth = 0.1
+                GL.glLineWidth(5)
+                GL.glColor4f(1.0, 0.0, 0.0, 1.0)
+                GL.glBegin(GL.GL_LINES)
+                GL.glVertex3f(self.selection[0] - 1, self.selection[1] - 1, depth)
+                GL.glVertex3f(self.selection[0] + 1, self.selection[1] + 1, depth)
+                GL.glVertex3f(self.selection[0] - 1, self.selection[1] + 1, depth)
+                GL.glVertex3f(self.selection[0] + 1, self.selection[1] - 1, depth)
+                GL.glEnd()
             else:
                 depth = self.project["setup"]["mill"]["depth"] - 0.1
                 GL.glLineWidth(5)
@@ -256,28 +271,39 @@ class GLWidget(QGLWidget):
 
     def toggle_tab_selector(self) -> None:
         self.selection = ()
-        self.tab_selector = not self.tab_selector
-        self.start_selector = False
-        self.repair_selector = False
-        if self.tab_selector:
+        if self.selector_mode == "":
+            self.selector_mode = "tab"
             self.view_2d()
+        else:
+            self.selector_mode = ""
+            self.view_reset()
 
     def toggle_start_selector(self) -> None:
         self.selection = ()
-        self.start_selector = not self.start_selector
-        self.tab_selector = False
-        self.repair_selector = False
-        if self.start_selector:
+        if self.selector_mode == "":
+            self.selector_mode = "start"
             self.view_2d()
+        else:
+            self.selector_mode = ""
+            self.view_reset()
 
     def toggle_repair_selector(self) -> None:
         self.selection = ()
-        self.repair_selector = not self.repair_selector
-        self.tab_selector = False
-        self.start_selector = False
-        if self.repair_selector:
-            self.update_drawing()
+        if self.selector_mode == "":
+            self.selector_mode = "repair"
             self.view_2d()
+        else:
+            self.selector_mode = ""
+            self.view_reset()
+
+    def toggle_delete_selector(self) -> None:
+        self.selection = ()
+        if self.selector_mode == "":
+            self.selector_mode = "delete"
+            self.view_2d()
+        else:
+            self.selector_mode = ""
+            self.view_reset()
 
     def view_2d(self) -> None:
         """toggle view function."""
@@ -289,11 +315,7 @@ class GLWidget(QGLWidget):
 
     def view_reset(self) -> None:
         """toggle view function."""
-        if self.tab_selector:
-            return
-        if self.start_selector:
-            return
-        if self.repair_selector:
+        if self.selector_mode != "":
             return
         self.ortho = False
         self.rot_x = -20.0
@@ -322,22 +344,22 @@ class GLWidget(QGLWidget):
         self.trans_x_last = self.trans_x
         self.trans_y_last = self.trans_y
         self.trans_z_last = self.trans_z
-        if (
-            self.tab_selector or self.start_selector or self.repair_selector
-        ) and self.selection:
+        if self.selector_mode != "" and self.selection:
             if self.mbutton == 1:
                 if self.selection:
-                    if self.tab_selector:
+                    if self.selector_mode == "tab":
                         self.project["tabs"]["data"].append(self.selection)
                         self.project["app"].update_tabs()
-                    elif self.start_selector:
+                    elif self.selector_mode == "start":
                         obj_idx = self.selection[2]
                         self.project["objects"][obj_idx]["start"] = (
                             self.selection[0],
                             self.selection[1],
                         )
                         self.project["app"].update_starts()
-                    elif self.repair_selector:
+                    elif self.selector_mode == "delete":
+                        pass
+                    elif self.selector_mode == "repair":
                         obj_idx = self.selection[2]
                         self.project["segments_org"].append(
                             {
@@ -355,7 +377,7 @@ class GLWidget(QGLWidget):
                 self.update_drawing()
                 self.update()
             elif self.mbutton == 2:
-                if self.tab_selector:
+                if self.selector_mode == "tab":
                     sel_idx = -1
                     sel_dist = -1
                     for tab_idx, tab in enumerate(self.project["tabs"]["data"]):
@@ -372,7 +394,12 @@ class GLWidget(QGLWidget):
                         self.update_drawing()
                         self.update()
                         self.project["app"].update_tabs()
-                elif self.start_selector:
+                elif self.selector_mode == "delete":
+                    obj_idx = self.selection[2]
+                    del self.project["objects"][obj_idx]
+                    self.update_drawing()
+                    self.update()
+                elif self.selector_mode == "start":
                     obj_idx = self.selection[2]
                     self.project["objects"][obj_idx]["start"] = ()
                     self.update_drawing()
@@ -415,21 +442,28 @@ class GLWidget(QGLWidget):
             moffset = self.mpos - event.pos()
             self.trans_x = self.trans_x_last + moffset.x() / self.screen_w
             self.trans_y = self.trans_y_last - moffset.y() / self.screen_h * self.aspect
-        elif self.tab_selector:
+        elif self.selector_mode == "tab":
             (self.mouse_pos_x, self.mouse_pos_y) = self.mouse_pos_to_real_pos(
                 event.pos()
             )
             self.selection = found_next_tab_point(
                 (self.mouse_pos_x, self.mouse_pos_y), self.project["offsets"]
             )
-        elif self.start_selector:
+        elif self.selector_mode == "start":
             (self.mouse_pos_x, self.mouse_pos_y) = self.mouse_pos_to_real_pos(
                 event.pos()
             )
             self.selection = found_next_segment_point(
                 (self.mouse_pos_x, self.mouse_pos_y), self.project["objects"]
             )
-        elif self.repair_selector:
+        elif self.selector_mode == "delete":
+            (self.mouse_pos_x, self.mouse_pos_y) = self.mouse_pos_to_real_pos(
+                event.pos()
+            )
+            self.selection = found_next_segment_point(
+                (self.mouse_pos_x, self.mouse_pos_y), self.project["objects"]
+            )
+        elif self.selector_mode == "repair":
             (self.mouse_pos_x, self.mouse_pos_y) = self.mouse_pos_to_real_pos(
                 event.pos()
             )
@@ -495,14 +529,58 @@ class ViaConstructor:
             "data": [],
             "table": None,
         },
+        "textwidget": None,
     }
     info = ""
     save_tabs = "no"
     save_starts = "no"
-    draw_reader: Union[DxfReader, SvgReader, HpglReader, StlReader, TtfReader]
+    draw_reader: Optional[DrawReaderBase] = None
+    status_bar: Optional[QWidget] = None
+    main: Optional[QMainWindow] = None
+
+    def save_objects_as_dxf(self, output_file) -> bool:
+        try:
+            doc = ezdxf.new("R2010")
+            msp = doc.modelspace()
+            doc.units = ezdxf.units.MM
+            for obj in self.project["objects"].values():
+                for segment in obj["segments"]:
+                    if segment["bulge"] == 0.0:
+                        msp.add_line(
+                            segment["start"],
+                            segment["end"],
+                            dxfattribs={"layer": segment["layer"]},
+                        )
+                    else:
+                        (
+                            center,
+                            start_angle,  # pylint: disable=W0612
+                            end_angle,  # pylint: disable=W0612
+                            radius,  # pylint: disable=W0612
+                        ) = ezdxf.math.bulge_to_arc(
+                            segment["start"], segment["end"], segment["bulge"]
+                        )
+                        msp.add_arc(
+                            center=center,
+                            radius=radius,
+                            start_angle=start_angle * 180 / math.pi,
+                            end_angle=end_angle * 180 / math.pi,
+                            dxfattribs={"layer": segment["layer"]},
+                        )
+            for vport in doc.viewports.get_config("*Active"):  # type: ignore
+                vport.dxf.grid_on = True
+                vport.dxf.center = (200, 200)
+            doc.saveas(output_file)
+        except Exception as error:  # pylint: disable=W0703
+            print(f"ERROR while saving sxf: {error}")
+            return False
+        return True
 
     def run_calculation(self) -> None:
         """run all calculations."""
+        if not self.draw_reader:
+            return
+
         psetup: dict = self.project["setup"]
         min_max = objects2minmax(self.project["objects"])
         self.project["minMax"] = min_max
@@ -549,9 +627,10 @@ class ViaConstructor:
             sys.exit(1)
 
         self.project["machine_cmd"] = polylines2machine_cmd(self.project, output_plugin)
-        self.project["textwidget"].clear()
-        self.project["textwidget"].insertPlainText(self.project["machine_cmd"])
-        self.project["textwidget"].verticalScrollBar().setValue(0)
+        if self.project["textwidget"]:
+            self.project["textwidget"].clear()
+            self.project["textwidget"].insertPlainText(self.project["machine_cmd"])
+            self.project["textwidget"].verticalScrollBar().setValue(0)
 
     def _toolbar_flipx(self) -> None:
         mirror_objects(self.project["objects"], self.project["minMax"], vertical=True)
@@ -589,6 +668,10 @@ class ViaConstructor:
         """tab selector."""
         self.project["glwidget"].toggle_tab_selector()
 
+    def _toolbar_toggle_delete_selector(self) -> None:
+        """delete selector."""
+        self.project["glwidget"].toggle_delete_selector()
+
     def _toolbar_toggle_start_selector(self) -> None:
         """start selector."""
         self.project["glwidget"].toggle_start_selector()
@@ -612,12 +695,18 @@ class ViaConstructor:
             return True
         return False
 
+    def status_bar_message(self, message) -> None:
+        if self.status_bar:
+            self.status_bar.showMessage(message)
+        else:
+            print(f"STATUS: {message}")
+
     def _toolbar_save_machine_cmd(self) -> None:
         """save machine_cmd."""
-        self.status_bar.showMessage(f"{self.info} - save machine_cmd..")
+        self.status_bar_message(f"{self.info} - save machine_cmd..")
         file_dialog = QFileDialog(self.main)
         file_dialog.setNameFilters(
-            [f"self.project['suffix'] (*.{self.project['suffix']})"]
+            [f"{self.project['suffix']} (*.{self.project['suffix']})"]
         )
         self.project[
             "filename_machine_cmd"
@@ -629,11 +718,44 @@ class ViaConstructor:
             f"{self.project['suffix']} (*.{self.project['suffix']})",
         )
         if name[0] and self.machine_cmd_save(name[0]):
-            self.status_bar.showMessage(
+            self.status_bar_message(
                 f"{self.info} - save maschine-code..done ({name[0]})"
             )
         else:
-            self.status_bar.showMessage(f"{self.info} - save maschine-code..cancel")
+            self.status_bar_message(f"{self.info} - save maschine-code..cancel")
+
+    def _toolbar_save_dxf(self) -> None:
+        """save doawing as dxf."""
+        self.status_bar_message(f"{self.info} - save drawing as dxf..")
+        file_dialog = QFileDialog(self.main)
+        file_dialog.setNameFilters(["dxf (*.dxf)"])
+        self.project[
+            "filename_machine_cmd"
+        ] = f"{'.'.join(self.project['filename_draw'].split('.')[:-1])}.dxf"
+        name = file_dialog.getSaveFileName(
+            self.main,
+            "Save File",
+            self.project["filename_machine_cmd"],
+            "dxf (*.dxf)",
+        )
+        if name[0] and self.save_objects_as_dxf(name[0]):
+            self.status_bar_message(f"{self.info} - save dxf..done ({name[0]})")
+        else:
+            self.status_bar_message(f"{self.info} - save dxf..cancel")
+
+    def _toolbar_load_drawing(self) -> None:
+        """load drawing."""
+        self.status_bar_message(f"{self.info} - load drawing..")
+        file_dialog = QFileDialog(self.main)
+        file_dialog.setNameFilters(["drawing (*.dxf)"])
+        name = file_dialog.getOpenFileName(
+            self.main, "Load Drawing", "", "drawing (*.dxf)"
+        )
+        if name[0] and self.load_drawing(name[0]):
+            self.update_drawing()
+            self.status_bar_message(f"{self.info} - load drawing..done ({name[0]})")
+        else:
+            self.status_bar_message(f"{self.info} - load drawing..cancel")
 
     def setup_load(self, filename: str) -> bool:
         if os.path.isfile(filename):
@@ -653,15 +775,15 @@ class ViaConstructor:
 
     def _toolbar_save_setup(self) -> None:
         """save setup."""
-        self.status_bar.showMessage(f"{self.info} - save setup..")
+        self.status_bar_message(f"{self.info} - save setup..")
         if self.setup_save(self.args.setup):
-            self.status_bar.showMessage(f"{self.info} - save setup..done")
+            self.status_bar_message(f"{self.info} - save setup..done")
         else:
-            self.status_bar.showMessage(f"{self.info} - save setup..error")
+            self.status_bar_message(f"{self.info} - save setup..error")
 
     def _toolbar_load_setup_from(self) -> None:
         """load setup from."""
-        self.status_bar.showMessage(f"{self.info} - load setup from..")
+        self.status_bar_message(f"{self.info} - load setup from..")
         file_dialog = QFileDialog(self.main)
         file_dialog.setNameFilters(["setup (*.json)"])
         name = file_dialog.getOpenFileName(
@@ -672,30 +794,29 @@ class ViaConstructor:
             self.update_table()
             self.global_changed(0)
             self.update_drawing()
-            self.status_bar.showMessage(
-                f"{self.info} - load setup from..done ({name[0]})"
-            )
+            self.status_bar_message(f"{self.info} - load setup from..done ({name[0]})")
         else:
-            self.status_bar.showMessage(f"{self.info} - load setup from..cancel")
+            self.status_bar_message(f"{self.info} - load setup from..cancel")
 
     def _toolbar_save_setup_as(self) -> None:
         """save setup as."""
-        self.status_bar.showMessage(f"{self.info} - save setup as..")
+        self.status_bar_message(f"{self.info} - save setup as..")
         file_dialog = QFileDialog(self.main)
         file_dialog.setNameFilters(["setup (*.json)"])
         name = file_dialog.getSaveFileName(
             self.main, "Save Setup", self.args.setup, "setup (*.json)"
         )
         if name[0] and self.setup_save(name[0]):
-            self.status_bar.showMessage(
-                f"{self.info} - save setup as..done ({name[0]})"
-            )
+            self.status_bar_message(f"{self.info} - save setup as..done ({name[0]})")
         else:
-            self.status_bar.showMessage(f"{self.info} - ave setup as..cancel")
+            self.status_bar_message(f"{self.info} - ave setup as..cancel")
 
     def update_drawing(self, draw_only=False) -> None:
         """update drawings."""
-        self.status_bar.showMessage(f"{self.info} - calculate..")
+        if not self.draw_reader:
+            return
+
+        self.status_bar_message(f"{self.info} - calculate..")
         if not draw_only:
             self.run_calculation()
         self.project["gllist"] = GL.glGenLists(1)
@@ -704,24 +825,35 @@ class ViaConstructor:
         if self.project["setup"]["view"]["3d_show"]:
             if hasattr(self.draw_reader, "draw_3d"):
                 self.draw_reader.draw_3d()
-        if not self.project["glwidget"].repair_selector:
+        if (
+            self.project["glwidget"]
+            and self.project["glwidget"].selector_mode != "repair"
+        ):
             if not draw_maschinecode_path(self.project):
-                self.status_bar.showMessage(
+                self.status_bar_message(
                     f"{self.info} - error while drawing maschine commands"
                 )
 
         if self.project["setup"]["view"]["object_ids"]:
             draw_object_ids(self.project)
 
-        draw_object_edges(self.project)
+        selected = -1
+        if (
+            self.project["glwidget"]
+            and self.project["glwidget"].selector_mode == "delete"
+        ):
+            selected = self.project["glwidget"].selection[2]
+
+        draw_object_edges(self.project, selected=selected)
         if self.project["setup"]["view"]["polygon_show"]:
             draw_object_faces(self.project)
         GL.glEndList()
         self.info = f"{self.project['minMax'][2] - self.project['minMax'][0]}x{self.project['minMax'][3] - self.project['minMax'][1]}mm"
-        self.main.setWindowTitle(
-            f"viaConstructor: {self.project['filename_draw']} - {self.info}"
-        )
-        self.status_bar.showMessage(f"{self.info} - calculate..done")
+        if self.main:
+            self.main.setWindowTitle(
+                f"viaConstructor: {self.project['filename_draw']} - {self.info}"
+            )
+        self.status_bar_message(f"{self.info} - calculate..done")
 
     def materials_select(self, material_idx) -> None:
         """calculates the milling feedrate and tool-speed for the selected material
@@ -762,7 +894,7 @@ class ViaConstructor:
         )
         info_test.append("")
         ret = QMessageBox.question(
-            self.main,
+            self.main,  # type: ignore
             "Warning",
             "\n".join(info_test),
             QMessageBox.Ok | QMessageBox.Cancel,
@@ -928,7 +1060,7 @@ class ViaConstructor:
             info_test.append(" will be removed !")
             info_test.append("")
             ret = QMessageBox.question(
-                self.main,
+                self.main,  # type: ignore
                 "Warning",
                 "\n".join(info_test),
                 QMessageBox.Yes | QMessageBox.No,
@@ -953,7 +1085,7 @@ class ViaConstructor:
             info_test.append(" will be removed !")
             info_test.append("")
             ret = QMessageBox.question(
-                self.main,
+                self.main,  # type: ignore
                 "Warning",
                 "\n".join(info_test),
                 QMessageBox.Yes | QMessageBox.No,
@@ -962,7 +1094,7 @@ class ViaConstructor:
                 self.save_tabs = "yes"
             else:
                 self.save_tabs = "no"
-        if self.save_tabs == "yes":
+        if self.save_tabs == "yes" and self.draw_reader:
             self.draw_reader.save_tabs(self.project["tabs"]["data"])
 
     def global_changed(self, value) -> None:  # pylint: disable=W0613
@@ -1030,7 +1162,7 @@ class ViaConstructor:
             "filename_machine_cmd"
         ] = f"{'.'.join(self.project['filename_draw'].split('.')[:-1])}.{self.project['suffix']}"
         if os.path.isfile(self.project["filename_machine_cmd"]):
-            self.status_bar.showMessage(
+            self.status_bar_message(
                 f"{self.info} - loading setup from maschinecode: {self.project['filename_machine_cmd']}"
             )
             with open(self.project["filename_machine_cmd"], "r") as fd_machine_cmd:
@@ -1042,11 +1174,11 @@ class ViaConstructor:
                         for sname in self.project["setup"]:
                             self.project["setup"][sname].update(ndata.get(sname, {}))
                         self.update_drawing()
-                        self.status_bar.showMessage(
+                        self.status_bar_message(
                             f"{self.info} - loading setup from maschinecode..done"
                         )
                         return
-        self.status_bar.showMessage(
+        self.status_bar_message(
             f"{self.info} - loading setup from maschinecode..failed"
         )
 
@@ -1057,6 +1189,22 @@ class ViaConstructor:
     def create_toolbar(self) -> None:
         """creates the_toolbar."""
         toolbuttons = {
+            _("Load drawing"): (
+                "open.png",
+                "",
+                _("Load drawing"),
+                self._toolbar_load_drawing,
+                True,
+                "main",
+            ),
+            _("Save drawing as DXF"): (
+                "save.png",
+                "Ctrl+S",
+                _("Save drawing as DXF"),
+                self._toolbar_save_dxf,
+                True,
+                "main",
+            ),
             _("Exit"): (
                 "exit.png",
                 "Ctrl+Q",
@@ -1162,7 +1310,7 @@ class ViaConstructor:
                 "tabs",
             ),
             _("Start-Selector"): (
-                "tab-selector.png",
+                "start.png",
                 "",
                 _("Start-Selector"),
                 self._toolbar_toggle_start_selector,
@@ -1170,20 +1318,30 @@ class ViaConstructor:
                 "start",
             ),
             _("Repair-Selector"): (
-                "tab-selector.png",
+                "repair.png",
                 "",
                 _("Repair-Selector"),
                 self._toolbar_toggle_repair_selector,
                 True,
                 "repair",
             ),
+            _("Delete-Selector"): (
+                "delete.png",
+                "",
+                _("Delete-Selector"),
+                self._toolbar_toggle_delete_selector,
+                True,
+                "delete",
+            ),
         }
         self.toolbar = QToolBar("top toolbar")
-        self.main.addToolBar(self.toolbar)
+        self.main.addToolBar(self.toolbar)  # type: ignore
         section = ""
 
         for title, toolbutton in toolbuttons.items():
             icon = os.path.join(self.this_dir, "..", "data", toolbutton[0])
+            if not os.path.isfile(icon):
+                icon = os.path.join("/usr", "data", toolbutton[0])
             if not os.path.isfile(icon):
                 icon = os.path.join("/usr", "local", "data", toolbutton[0])
 
@@ -1355,13 +1513,57 @@ class ViaConstructor:
 
         self.project["maxOuter"] = find_tool_offsets(self.project["objects"])
 
+    def load_drawing(self, filename: str) -> bool:
+        # clean project
+        self.project["filename_draw"] = ""
+        self.project["filename_machine_cmd"] = ""
+        self.project["suffix"] = "ngc"
+        self.project["axis"] = ["X", "Y", "Z"]
+        self.project["machine_cmd"] = ""
+        self.project["segments"] = {}
+        self.project["objects"] = {}
+        self.project["offsets"] = {}
+        self.project["gllist"] = []
+        self.project["maxOuter"] = []
+        self.project["minMax"] = []
+        self.project["table"] = []
+        self.project["status"] = "INIT"
+        self.project["tabs"] = {"data": [], "table": None}
+        self.info = ""
+        self.save_tabs = "no"
+        self.save_starts = "no"
+        self.draw_reader = None
+
+        # find plugin
+        suffix = filename.split(".")[-1].lower()
+        for reader_plugin in reader_plugins.values():
+            if suffix in reader_plugin.suffix():
+                self.draw_reader = reader_plugin(filename, self.args)
+                if reader_plugin.can_save_tabs:
+                    self.save_tabs = "ask"
+                break
+
+        if self.draw_reader:
+            self.project["segments_org"] = self.draw_reader.get_segments()
+            self.project["filename_draw"] = filename
+            self.project[
+                "filename_machine_cmd"
+            ] = f"{'.'.join(self.project['filename_draw'].split('.')[:-1])}.{self.project['suffix']}"
+            self.prepare_segments()
+            return True
+
+        print(f"ERROR: can not load file: {filename}")
+        return False
+
     def __init__(self) -> None:
         """viaconstructor main init."""
         setproctitle.setproctitle("viaconstructor")  # pylint: disable=I1101
 
         # arguments
         parser = argparse.ArgumentParser()
-        parser.add_argument("filename", help="input file", type=str)
+        parser.add_argument(
+            "filename", help="input file", type=str, nargs="?", default=None
+        )
         parser.add_argument(
             "-s",
             "--setup",
@@ -1370,7 +1572,18 @@ class ViaConstructor:
             default=f"{os.path.join(Path.home(), 'viaconstructor.json')}",
         )
         parser.add_argument(
-            "-o", "--output", help="save to machine_cmd", type=str, default=None
+            "-o",
+            "--output",
+            help="save to machine_cmd and exit",
+            type=str,
+            default=None,
+        )
+        parser.add_argument(
+            "-d",
+            "--dxf",
+            help="convert drawing to dxf file and exit",
+            type=str,
+            default=None,
         )
         # STL-Reader
         parser.add_argument(
@@ -1404,37 +1617,18 @@ class ViaConstructor:
             self.setup_load(self.args.setup)
 
         # load drawing #
-        if self.args.filename.lower().endswith(".svg"):
-            self.draw_reader = SvgReader(self.args.filename, self.args)
-
-        elif self.args.filename.lower().endswith(".eps"):
-            print("# try to convert eps into dxf format")
-            with tempfile.NamedTemporaryFile() as tmp:
-                cmd = f"pstoedit -f dxf {self.args.filename} {tmp.name}"
-                print(f"# running: {cmd}")
-                os.system(cmd)
-                self.draw_reader = DxfReader(tmp.name, self.args)
-        elif self.args.filename.lower().endswith(".dxf"):
-            self.draw_reader = DxfReader(self.args.filename, self.args)
-            self.save_tabs = "ask"
-        elif self.args.filename.lower().endswith(".hpgl"):
-            self.draw_reader = HpglReader(self.args.filename, self.args)
-        elif self.args.filename.lower().endswith(".stl"):
-            self.draw_reader = StlReader(self.args.filename, self.args)
-        elif self.args.filename.lower().endswith(".ttf"):
-            self.draw_reader = TtfReader(self.args.filename, self.args)
-        else:
-            print(f"ERROR: Unknown file suffix: {self.args.filename}")
-            sys.exit(1)
-
-        self.project["segments_org"] = self.draw_reader.get_segments()
-        self.project["filename_draw"] = self.args.filename
-        self.project[
-            "filename_machine_cmd"
-        ] = f"{'.'.join(self.project['filename_draw'].split('.')[:-1])}.{self.project['suffix']}"
-
-        # prepare #
-        self.prepare_segments()
+        if self.args.filename and self.load_drawing(self.args.filename):
+            # save and exit
+            if self.args.dxf:
+                self.update_drawing()
+                print("saving dawing to file:", self.args.dxf)
+                self.save_objects_as_dxf(self.args.dxf)
+                sys.exit(0)
+            if self.args.output:
+                self.update_drawing()
+                print("saving machine_cmd to file:", self.args.output)
+                open(self.args.output, "w").write(self.project["machine_cmd"])
+                sys.exit(0)
 
         # gui #
         qapp = QApplication(sys.argv)
@@ -1464,7 +1658,7 @@ class ViaConstructor:
 
         self.status_bar = QStatusBar()
         self.main.setStatusBar(self.status_bar)
-        self.status_bar.showMessage(f"{self.info} - startup")
+        self.status_bar_message(f"{self.info} - startup")
 
         self.project["textwidget"] = QPlainTextEdit()
         self.project["objwidget"] = QTreeView()
@@ -1508,14 +1702,9 @@ class ViaConstructor:
         hlay.addWidget(left_widget, stretch=1)
         hlay.addWidget(right_widget, stretch=3)
 
-        if self.args.output:
-            self.update_drawing()
-            print("saving machine_cmd to file:", self.args.output)
-            open(self.args.output, "w").write(self.project["machine_cmd"])
-        else:
-            self.main.resize(1600, 1200)
-            self.main.show()
-            sys.exit(qapp.exec_())
+        self.main.resize(1600, 1200)
+        self.main.show()
+        sys.exit(qapp.exec_())
 
 
 if __name__ == "__main__":
